TEST_COLLECTION_ID = "collection-filter-test"
PLONE_PATH = "/plone"
COLLECTION_TEST_QUERY = (
    '[{"i": "path", "o": "plone.app.querystring.operation.string.path", "v": "%s"}]'
    % (PLONE_PATH)
)  # noqa
